# Generated by Django 3.1.1 on 2021-01-18 11:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("nuntius", "0019_add_tracking_index"),
    ]

    operations = [
        migrations.AlterField(
            model_name="campaign",
            name="utm_name",
            field=models.CharField(
                blank=True,
                help_text="Value used as utm_campaign parameter, used by various analytics tools.",
                max_length=255,
                verbose_name="UTM name (visible to subscribers)",
            ),
        ),
        migrations.AddIndex(
            model_name="campaignsentevent",
            index=models.Index(
                fields=["subscriber", "datetime"], name="nuntius_cam_subscri_f43488_idx"
            ),
        ),
    ]
