# Generated by Django 2.2.5 on 2020-01-28 09:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("nuntius", "0016_indices_and_ordering")]

    operations = [
        migrations.AddField(
            model_name="campaign",
            name="utm_name",
            field=models.CharField(
                blank=True,
                help_text="Name used as utm_campaign parameter, used by various analytics tools.",
                max_length=255,
                verbose_name="UTM name (visible to subscribers)",
            ),
        )
    ]
